----------------------------
--      Declaration       --
----------------------------

local AceOO = AceLibrary("AceOO-2.0")
local BS = AceLibrary("Babble-Spell-2.2")
local _G = getfenv(0)

GotWood.circle = AceOO.Class(GotWood.abstractview, "AceDB-2.0", "AceEvent-2.0")

local defaults = {
	skin = "Serenity",
}

-- Skin and code provided by Sairen, thx goes to him for this nice view!
local skins = {
	["Serenity"] = {
		["icon"] 	= { w=27, h=27, x=6,  y=-6, },
		["overlay"] = { w=43, h=43, x=-8, y=8,  tex="Interface\\AddOns\\GotWood\\images\\serenity0" },
		["flash"]	= { w=30, h=30, x=3,  y=-3, tex="Interface\\AddOns\\GotWood\\images\\overlayred" },
		["cooldown"]= { w=30, h=30, x=0,  y=0, },
	},
}

GotWood:RegisterDefaults("circle", "profile", defaults)

----------------------------
--      Main Functions    --
----------------------------

function GotWood.circle.prototype:init(totemOrder)
	-- Call Superclass
	GotWood.circle.super.prototype.init(self)

	-- Save the totem order
	self.totemOrder = totemOrder
	self.totemLife = {}
	self.circles = {}

	-- Acquire db namespace
	self.db = GotWood:AcquireDBNamespace("circle")

	self:CreateCircles()
end

function GotWood.circle.prototype:CreateCircles()
	local icon = skins[self.db.profile.skin].icon
	for k,v in ipairs(self.totemOrder) do
		self.circles[v] = {}
		self.circles[v].frame = CreateFrame("Button", "GotWoodCircle"..v, UIParent)
		self.circles[v].frame:SetWidth(icon.w)
		self.circles[v].frame:SetHeight(icon.h)
		self:SkinOverlay(self.circles[v].frame)
		self:AddCooldownDisplay(self.circles[v].frame)

		self.circles[v].time = self.circles[v].frame:CreateFontString("$parentTime","ARTWORK","GameFontNormalSmall")
		self.circles[v].time:SetWidth(36)
		self.circles[v].time:SetHeight(10)
		self.circles[v].time:SetTextColor(1,0,0)
		self.circles[v].time:SetPoint("TOP", self.circles[v].frame, "TOP", -2, 10)
		self.circles[v].time:SetDrawLayer("OVERLAY")
		local font = self.circles[v].time:GetFont()
		self.circles[v].time:SetFont(font, 15)
		self.circles[v].time:Show()

		self.circles[v].frame:Hide()
	end
end

function GotWood.circle.prototype:AddCooldownDisplay(circle)
	local skin = skins[self.db.profile.skin].cooldown
	local cooldown = CreateFrame("Model", "$parentCooldown", circle, "CooldownFrameTemplate")
	cooldown:SetScale(skins[self.db.profile.skin].icon.w/37)
	cooldown:SetFrameLevel(1)
	cooldown:SetAllPoints(circle)
end

function GotWood.circle.prototype:SkinOverlay(circle)
	local skin = skins[self.db.profile.skin].overlay
	local overlay
	overlay = circle:CreateTexture("$parentOverlay", "OVERLAY")
	overlay:SetTexture(skin.tex)
	overlay:SetWidth(skin.w)
	overlay:SetHeight(skin.w)
	overlay:SetPoint("TOPLEFT", circle, "TOPLEFT", skin.x, skin.y)
end

function GotWood.circle.prototype:SkinIcon(name)
	local skin = skins[self.db.profile.skin].icon
	_G[name]:SetWidth(skin.w)
	_G[name]:SetHeight(skin.h)
end

function GotWood.circle.prototype:SetTotem(tName, tRank)
	-- Get the ele type of the totem
	local ele = GotWoodData[tName].ele

	-- Set the texture
	local frameName = self.circles[ele].frame:GetName()
	self.circles[ele].frame:SetNormalTexture(BS:GetSpellIcon(tName) or GotWoodData[tName].icon)
	self:SkinIcon(frameName)
	
	-- Set the time
	local time
	if tRank and GotWoodData[tName][tRank] then
		time = GotWoodData[tName][tRank].duration
	else
		time = GotWoodData[tName].duration
	end
	CooldownFrame_SetTimer(_G[frameName.."Cooldown"], GetTime(), time, 1)
	self.circles[ele].frame:SetAlpha(1)
	self.circles[ele].time:SetText(time)
	self.circles[ele].timeleft = time-1

	-- Store the life in our local table
	self.totemLife[ele] = {["name"] = tName, ["life"] = GotWoodData[tName].life}

	-- Start the timer
	self.circles[ele].frame:Show()
	if self.circles[ele].timer and self:IsEventScheduled(self.circles[ele].timer) then
		self:CancelScheduledEvent(self.circles[ele].timer)
	end
	self.circles[ele].timer = self:ScheduleRepeatingEvent(self.UpdateTime, 1, self, ele)
end

function GotWood.circle.prototype:UpdateTime(ele)
	local time = self.circles[ele].timeleft
	if time > 0 then
		self.circles[ele].timeleft = time - 1
		self.circles[ele].time:SetText(time)
	else
		self:CancelScheduledEvent(self.circles[ele].timer)
		self.circles[ele].frame:SetAlpha(0.5)
		self.circles[ele].time:SetText("")
	end
end

function GotWood.circle.prototype:RemoveTotem(ele)
	if self.circles[ele].timer and self:IsEventScheduled(self.circles[ele].timer) then
		self:CancelScheduledEvent(self.buttons[ele].timer)
	end
	self.circles[ele].frame:SetAlpha(0.5)
	self.circles[ele].time:SetText("")
end

function GotWood.circle.prototype:SetTotemDamage(tName, dmg)
	-- Search for the correct button
	for ele,v in pairs(self.totemLife) do
		if v.name == tName then
			v.life = v.life-dmg
			if v.life<0 then
				-- Stop timer
				if self.circles[ele].timer and self:IsEventScheduled(self.circles[ele].timer) then
					self:CancelScheduledEvent(self.circles[ele].timer)
				end
				self.circles[ele].frame:SetAlpha(0.5)
				self.circles[ele].time:SetText("")
				self.totemLife[ele] = nil
			end
			break
		end
	end
end

function GotWood.circle.prototype:UpdateOrder()
	-- Set the positions
	for k,v in ipairs(self.totemOrder) do
		if k==1 then
			self.circles[v].frame:SetPoint("TOPLEFT", GotWood.frame, "TOPLEFT", 10, -22)
		else
			-- Attach the button to the previous one
			self.circles[v].frame:SetPoint("TOPLEFT", self.circles[self.totemOrder[k-1]].frame, "TOPRIGHT", 12, 0)
		end
	end
end

--[[function GotWood.circle.prototype:GetOptions()
	return self.options
end]]

function GotWood.circle.prototype:GetWidth()
	return 164
end

function GotWood.circle.prototype:GetHeight()
	return 65
end

function GotWood.circle.prototype:SetScale(scale)
	for k,v in ipairs(self.totemOrder) do
		self.circles[v].frame:SetScale(scale)
	end
end

function GotWood.circle.prototype:UpdatePosition()
	self:UpdateOrder()
end

function GotWood.circle.prototype:Disable()
	for k,v in ipairs(self.totemOrder) do
		local name = self.circles[v].frame:GetName()
		if self.circles[v].timer and self:IsEventScheduled(self.circles[ele].timer) then
			self:CancelScheduledEvent(self.circles[v].timer)
		end
		self.circles[v].frame:Hide()
		_G[name.."Overlay"] = nil
		_G[name.."Cooldown"] = nil
		self.circles[v].frame = nil
		self.circles[v].time = nil
	end
end

-- Important: GotWood needs to know that this view exists, so add it to its table
-- The second abbreviation has to be the same name as the class name
GotWood:AddView("Circles", "circle")